local AddonName, a = ...
if a.BuildFail(50000) then return end
local L = a.Localize
local s = SpellFlashAddon
local c = BittensSpellFlashLibrary

local UnitCreatureType = UnitCreatureType
local UnitLevel = UnitLevel
local math = math

c.Init(a)

------------------------------------------------------------------------ common
c.AddOptionalSpell("Blessing of Kings", nil, { NoRangeCheck = 1 })

c.AddOptionalSpell("Blessing of Might", nil, { NoRangeCheck = 1 })

c.AddOptionalSpell("Holy Avenger", nil, { 
	NoGCD = true,
	CheckFirst = function()
		return a.HolyPower < 3
	end
})

c.AddSpell("Crusader Strike", nil, {
	NotIfActive = true,
	CheckFirst = function()
		return not c.IsCasting("Hammer of the Righteous")
	end,
})

c.AddSpell("Judgment", nil, {
	NotIfActive = true,
})

c.AddInterrupt("Rebuke")

-------------------------------------------------------------------------- Holy
local function hpCheckFirst(z)
	if a.HolyPower < 3 then
		return false
	end
	
	local dp = c.GetBuffDuration("Divine Purpose")
	if dp > 0 and dp < 2.5 then
		z.FlashSize = nil
		z.FlashColor = "red"
		return true
	end
	
	z.FlashColor = "yellow"
	if a.HolyPower < 5 then
		z.FlashSize = s.FlashSizePercent() / 2
	else
		z.FlashSize = nil
	end
	return true
end

c.AddOptionalSpell("Seal of Insight", nil, {
	Type = "form",
})

c.AddOptionalSpell("Word of Glory", "for Holy", {
	Override = hpCheckFirst,
})

c.AddOptionalSpell("Eternal Flame", "for Holy", {
	Override = hpCheckFirst,
})

c.AddOptionalSpell("Light of Dawn", "for Holy", {
	Override = hpCheckFirst,
})

c.AddOptionalSpell("Holy Shock", "under 5 with Daybreak", {
	NoRangeCheck = 1,
	CheckFirst = function()
		return a.HolyPower < 5 
			and (c.HasBuff("Daybreak") or c.IsCasting("Holy Radiance"))
	end
})

c.AddOptionalSpell("Beacon of Light", nil, {
	Override = function()
		return s.InRaidOrParty()
			and not s.MyBuff(c.GetID("Beacon of Light"), a.BeaconTarget)
	end
})

c.AddOptionalSpell("Sacred Shield", "for Holy", {
	NoRangeCheck = true,
	CheckFirst = function()
		return not s.MyBuff(c.GetID("Sacred Shield"), a.SacredShieldTarget)
	end
})

c.AddOptionalSpell("Flash of Light", "under Selfless Healer", { 
	NoRangeCheck = 1,
	CheckFirst = function()
		return c.GetBuffStack("Selfless Healer") == 3
	end
})

c.AddOptionalSpell("Flash of Light", "to Save Selfless Healer", {
	NoRangeCheck = 1,
	FlashColor = "red",
	CheckFirst = function()
		local duration = c.GetBuffDuration("Selfless Healer")
		return duration > 0 and duration < 2.5
	end
})

c.AddOptionalSpell("Judgment", "to Save Selfless Healer", {
	NoRangeCheck = 1,
	FlashColor = "red",
	CheckFirst = function()
		local duration = c.GetBuffDuration("Selfless Healer")
		return duration > 0 and duration < 2.5
	end
})

-------------------------------------------------------------------- protection
c.AddOptionalSpell("Seal of Insight", "for Prot", {
	Type = "form",
	CheckFirst = function()
		return not s.Form()
	end
})

c.AddOptionalSpell("Righteous Fury", nil, {
	NoGCD = true,
	Buff = "Righteous Fury",
	BuffUnit = "player",
})

c.AddOptionalSpell("Avenging Wrath", "if Plain", {
	NoGCD = true,
	CheckFirst = function()
		return not c.HasTalent("Sanctified Wrath")
			and not c.HasGlyph("Avenging Wrath")
	end
})

c.AddOptionalSpell("Avenging Wrath", "if Cool for Prot", {
	NoGCD = true,
	Enabled = function()
		return c.HasTalent("Sanctified Wrath") or c.HasGlyph("Avenging Wrath")
	end,
	ShouldHold = function()
		return c.HasGlyph("Avenging Wrath") and s.HealthPercent("player") > 85
	end
})

c.AddOptionalSpell("Word of Glory", "for Prot", {
	Override = function()
		return a.HolyPower > 0
			and c.GetCooldown("Word of Glory", true) == 0 
			and s.HealthDamagePercent("player") - 5 
				> math.min(3, a.HolyPower) 
					* 4 
					* (1 + .1 * c.GetBuffStack("Bastion of Glory", true))
	end
})

c.AddOptionalSpell("Divine Protection", nil, {
	NoGCD = true,
	CheckFirst = function()
		return c.HasGlyph("Divine Protection")
	end
})

c.AddOptionalSpell("Holy Avenger", "for Prot", { 
	NoGCD = true,
	ShouldHold = function()
		return a.HolyPower >= 3
	end
})

c.AddOptionalSpell("Guardian of Ancient Kings", nil, {
	NoGCD = true,
	NoRangeCheck = true,
})

c.AddOptionalSpell("Ardent Defender", nil, {
	NoGCD = true,
	Enabled = function()
		return not c.WearingSet(2, "ProtT14")
	end
})

c.AddOptionalSpell("Ardent Defender", "2pT14", {
	NoGCD = true,
	Enabled = function()
		return c.WearingSet(2, "ProtT14")
	end
})

c.AddOptionalSpell("Shield of the Righteous")

c.AddOptionalSpell("Shield of the Righteous", "to save Buffs", {
	Override = function()
		if a.HolyPower < 3 or not s.MeleeDistance() then
			return false
		end
		
		local duration = c.GetBuffDuration("Bastion of Glory")
		if duration > 0 and duration < 1.6 then
			return true
		end
		
		local duration = c.GetBuffDuration("Divine Purpose")
		if duration > 0 and duration < 1.6 then
			return true
		end
	end
})

c.AddSpell("Holy Wrath", nil, {
	Melee = true,
	NoRangeCheck = true,
	NotIfActive = true,
})

c.AddSpell("Holy Wrath", "to Stun", {
	Melee = true,
	NotIfActive = true,
	NoRangeCheck = true,
	CheckFirst = function()
		local level = UnitLevel("target")
		if level == -1 or level > UnitLevel("player") then
			return false
		end
		
		local ct = UnitCreatureType("target")
		if ct == L["Demon"] or ct == L["Undead"] then
			return true
		end
		
		if not c.HasGlyph("Holy Wrath") then
			return false
		end
		
		return ct == L["Dragonkin"] or ct == L["Elemental"]
	end
})

c.AddSpell("Hammer of the Righteous", "for Debuff", {
	Debuff = c.WEAKENED_BLOWS_DEBUFFS,
	CheckFirst = function(z)
		z.EarlyRefresh = c.GetHastedTime(4.5)
		return not c.IsCasting("Crusader Strike")
	end
})

c.AddSpell("Eternal Flame", "for Prot", {
	EarlyRefresh = 1, -- overwritten once applied
	CheckFirst = function(z)
		return a.HolyPower >= 3 
			and c.GetBuffDuration("Eternal Flame") < z.EarlyRefresh
	end
})
c.ManageDotRefresh("Eternal Flame", 3)

c.AddSpell("Sacred Shield", nil, {
	Buff = "Sacred Shield",
	BuffUnit = "player",
	NoRangeCheck = true,
	NotIfActive = true,
})

c.AddSpell("Sacred Shield", "Refresh", {
	NoRangeCheck = true,
	NotIfActive = true,
	CheckFirst = function()
		return c.GetBuffDuration("Sacred Shield") < 5
	end
})

c.AddSpell("Flash of Light", "for Prot", {
	NoRangeCheck = true,
	CheckFirst = function()
		return a.SelflessHealer == 3 and s.HealthPercent("player") < 85
	end
})

c.AddSpell("Avenger's Shield", nil, {
	NotIfActive = true,
})

c.AddSpell("Avenger's Shield", "under Grand Crusader", {
	NotIfActive = true,
	CheckFirst = function()
		return c.HasBuff("Grand Crusader")
	end
})

c.AddOptionalSpell("Consecration", nil, {
	NoRangeCheck = true,
	CheckFirst = function(z)
		z.Melee = not c.HasGlyph("Consecration")
		return c.GetCooldown("Consecration Glyphed") == 0
	end
})

c.AddTaunt("Hand of Reckoning", nil, { NoGCD = true })

------------------------------------------------------------------- retribution
c.AddOptionalSpell("Avenging Wrath", nil, { 
	NoGCD = true,
	CheckFirst = function()
		return c.HasBuff("Inquisition")
	end
})

c.AddSpell("Inquisition", nil, {
	Override = function()
		return a.HolyPower >= 3
			and c.GetBuffDuration("Inquisition") < 2
			and not c.IsCasting("Inquisition")
	end
})

c.AddSpell("Templar's Verdict", nil, { 
	Override = function()
		return a.HolyPower >= 3
	end
})

c.AddSpell("Templar's Verdict", "at 5", { 
	Override = function()
		return a.HolyPower == 5
	end
})

c.AddSpell("Hammer of Wrath", "for Ret", {
	Override = function(z)
		if (s.HealthPercent() > 20 and not c.HasBuff("Avenging Wrath"))
			or c.IsCasting("Hammer of Wrath") then
			return false
		end
		
		if c.IsCasting("Hammer of Wrath") then
			return false
		end
		
		local cd = c.GetCooldown("Hammer of Wrath")
		if cd == 0 then
			z.FlashColor = nil
			z.FlashSize = nil
			return true
		elseif z.FlashColor == nil then
			z.FlashColor = "green"
			z.FlashSize = s.FlashSizePercent() / 2
		end
		return cd < .2
	end
})

c.AddSpell("Exorcism", nil, {
	Run = function(z)
		if c.HasGlyph("Mass Exorcism") then
			z.Melee = true
			z.ID = c.GetID("Glyphed Exorcism")
		else
			z.Melee = nil
			z.ID = c.GetID("Exorcism")
		end
	end
})

c.AddOptionalSpell("Seal of Insight", "below 20", {
	Type = "form",
	CheckFirst = function()
		return s.PowerPercent("player") < 20
	end
})

c.AddOptionalSpell("Seal of Truth", nil, {
	Type = "form",
	CheckFirst = function()
		return s.Form() == nil
			or (s.Form("Seal of Insight") and s.PowerPercent("player") > 90)
	end
})
